/**************
Debugger Strip GH, a simple demonstration of how to write a 
control strip module. 8/20/94

In case you can't use the enclosed CodeWarrior project, you should 
use this code to compile a code resource of type 'sdev' and resID 0, 
and merge it into the enclosed control strip module.

This module simply drops into MacsBug when clicked.
Command clicking brings up a menu of ways to drop into Macsbug. You
can add different ways by adding items to the appropriate STR# resources
(id 998 = menu itms and id 999 = debugger strings).

Read the Software section of the Developer Notes on the Powerbook 500 series.

I am unable to include the file "controlstrip.h", but it should be available
in a variety of places.it is in the current Universal Header distribution (ETO 15) 
and should be available from Think or MetroWerks. (It isn't in MetroWerks CW4). 
You might check ftp.apple.com to see if there is an up-to-date Universal Header
distribution there. There is an unofficial version that comes with the Control 
Strip Patcher

Problems I've noticed:
 * SBTrackSlider appears to be broken; it doesn't return the correct value
 * SBGetDetachedIndString doesn't necessarily return a null string if the 
   index is invalid
 * Think Reference has incorrect declarations for the icon suite routines
   Look for the official Apple Icons.h file.
 * This module doesn't dehilite if the user drags outside its rect like 
   it ought to. I'm thinking of the proper way to do this.
 
--glenn howes
grhowes@students.wisc.edu
*****************/

#include "debug.h"
#include <Icons.h>
#include "controlstrip.h"



pascal long main (long message, long params, Rect *statusRect, GrafPtr statusPort)
{
	long result = 0L;

	switch (message)
	{
		case sdevInitModule: // check environs, allocate globals
			result = DoCSInit(); // returns are allocated globals
		break;
		case sdevCloseModule: // release my memory
			DoCSClose((MyGlobalHandle) params);
		break;
		case sdevFeatures: // I want to handle mouseclicks
			result = (1L << sdevWantMouseClicks) | (1L << sdevDontAutoTrack);
		break;
		case sdevGetDisplayWidth:
			result = 16L; // size of a small icon width
		break;
		case sdevPeriodicTickle: // idle routine
		break;
		case sdevDrawStatus: // draw my icon
			DoCSDraw((MyGlobalHandle) params, statusRect, statusPort);
		break;
		case sdevMouseClick: // the mouse was clicked in my button
			result = DoCSClick((MyGlobalHandle) params, statusRect, statusPort);
		break;
		case sdevSaveSettings: // DoCSClick must have returned a 2 sometime
			DoCSSavePrefs((MyGlobalHandle) params);
		break;
		case sdevShowBalloonHelp:
		break;
		
	}
	return (result);
}
/******
If I returned a value with the #1 bit set (a 2 for instance) in response to
a sdevPeriodicTickle or sdevMouseClick, I will receive this message

*********/
void DoCSSavePrefs(MyGlobalHandle myGlobals)
{
	Str255		aString;
	OSErr		iErr;
	Handle		prefH;
	if (prefH = NewHandle(sizeof(short)))
	{
		SBGetDetachedIndString(aString, (*myGlobals)->utilityStrings, kPrefName);
		**(short**)prefH = (*myGlobals)->whichString;
		iErr = SBSavePreferences(aString, prefH);
		DisposeHandle(prefH);
	}
}
long DoCSClick(MyGlobalHandle myGlobals, Rect *statusRect, GrafPtr statusPort)
{
	Boolean 	shiftPressed, commandPressed, inMyRect=true;
	Point		localPnt;
	long		retVal = 0L;
	short		oldIndex, newIndex;
	MenuHandle	hMenu;
	Str255		debugString;
	
	shiftPressed =  IsPressed(kShiftKey);
	commandPressed = IsPressed(kCommandKey);
	
	if (commandPressed) // present a configuration menu
	{
		hMenu = MakePrefsMenu(myGlobals);
		if (hMenu)
		{
			oldIndex = (*myGlobals)->whichString;
			newIndex = SBTrackPopupMenu(statusRect, hMenu);
			if (newIndex && newIndex != oldIndex)
			{
				(*myGlobals)->whichString = newIndex;
				retVal = (1L<< sdevNeedToSave);// set preference changed flag
			}
			DisposeMenu(hMenu);
		}
		if (!shiftPressed || !newIndex) return (retVal);
		// else execute immediately
	}
	else // manually track mouse
	{
		do
		{
			GetMouse(&localPnt);
			inMyRect = PtInRect(localPnt,statusRect);
		}while (StillDown());
	}
	if (inMyRect)
	{
		SBGetDetachedIndString(debugString, 
				(*myGlobals)->debuggerStrings, (*myGlobals)->whichString);
		
		if (debugString[0])
			DebugStr(debugString);
		else
			Debugger();
	}
	return (retVal);
}

MenuHandle MakePrefsMenu(MyGlobalHandle myGlobals)
{
	MenuHandle	retVal = 0L;
	Str255		menuItem;
	Str63		dummy;
	short		item = 1, numItems;
	short		**numItemsH;
	dummy[0] = 0;
	retVal = NewMenu(702,dummy);
	
	if (retVal)
	{	
		numItemsH = (short**) (*myGlobals)->menuStrings;
		numItems = **numItemsH;
		for(item = 1; item<= numItems; item++)
		{
			SBGetDetachedIndString(menuItem, (*myGlobals)->menuStrings, item);

			AppendMenu(retVal,menuItem);
		}
		if ((*myGlobals)->whichString <= numItems)
			SetItemMark(retVal,(*myGlobals)->whichString,sdevMenuItemMark); // the bullet
	}
	return(retVal);
}
long DoCSInit(void)
{
	MyGlobalHandle	myH;
	OSErr			iErr;
	Handle			iconSuite = 0L;
	Handle			menuStrings, debugStrings, utilityStrings;
	Str255			prefName;
	short			**whichStringH;
	if (HasDebugger()) // is a debugger installed?
	{
		myH = (MyGlobalHandle) NewHandleClear(sizeof(MyGlobals));
		iErr = SBGetDetachIconSuite(&iconSuite, 669, 0x0000FF00L);
		menuStrings = GetDetachedStrings(kMenuStrings);
		debugStrings = GetDetachedStrings(kDebuggerStrings);
		utilityStrings = GetDetachedStrings(kUtilityStrings);
		
		if (myH&&iconSuite &&menuStrings && debugStrings && utilityStrings) 
		// was I able to allocate my memory?
		{
			(*myH)->menuStrings = menuStrings;
			(*myH)->debuggerStrings = debugStrings;
			(*myH)->utilityStrings = utilityStrings;
			(*myH)->iconSuite = iconSuite;
			
			SBGetDetachedIndString(prefName, utilityStrings, kPrefName);
			iErr = SBLoadPreferences(prefName, (Handle*) &whichStringH);
			if (!iErr && whichStringH)
			{
				(*myH)->whichString = **whichStringH;
				DisposeHandle((Handle)whichStringH);
			}
			else (*myH)->whichString = 1; // default
			
			return ((long) myH); // return my globals
		}
		else // failed to allocate all my needed memory
		{
			if(myH) DisposeHandle((Handle) myH);
			if(iconSuite) DisposeIconSuite (iconSuite, true);
			if (menuStrings) DisposeHandle(menuStrings);
			if (debugStrings) DisposeHandle(debugStrings);
			if (utilityStrings) DisposeHandle(utilityStrings);
		}
	}
	return (-1L); // something went wrong, don't install
}
void DoCSClose(MyGlobalHandle myGlobals)
{
	if (myGlobals) 
	{
	 	DisposeIconSuite ((*myGlobals)->iconSuite, true);
		DisposeHandle((*myGlobals)->menuStrings);
		DisposeHandle((*myGlobals)->debuggerStrings);
		DisposeHandle((*myGlobals)->utilityStrings);
		DisposeHandle((Handle) myGlobals);
	}
}
void DoCSDraw(MyGlobalHandle myGlobals, Rect *statusRect, GrafPtr statusPort)
{
	OSErr		iErr;
	Rect		theRect;
	short		alignment = 0x01 + 0x04; // center up & down , and left & right
	long		transform = 0; // no transform
	
	if ((*myGlobals)->iconSuite)
	{
		theRect.top = statusRect->top;
		theRect.left = statusRect->left;
		theRect.bottom = statusRect->bottom;
		theRect.right = statusRect->right;
		iErr = PlotIconSuite(&theRect,alignment,transform,(*myGlobals)->iconSuite);
	}
}
Boolean HasDebugger(void)
{
	OSErr			err;
	long			*jmpAddress;
	
	jmpAddress = (long *) 0x120L; // address of entry routine for a debugger
	
	if (*jmpAddress) 
		return (true);
	else
		return (false);
}
Handle GetDetachedStrings(short id)
{
	Handle		strH;
	strH = Get1Resource('STR#',id);
	if (strH)
	{
		DetachResource(strH);
	}
	return(strH);
}
Boolean IsPressed(unsigned short key)
{// find out if the key is down (used to find if option key is down on cancel)
	unsigned char keyMap[16];
	GetKeys((long *)&keyMap);
	return((keyMap[key>>3]>>(key & 7)) & 1);
}